package BookDB::Schema::DB;

use strict;
use warnings;

use base 'DBIx::Class::Schema';

__PACKAGE__->load_classes;


# Created by DBIx::Class::Schema::Loader v0.04003 @ 2008-01-14 14:54:11
# DO NOT MODIFY THIS OR ANYTHING ABOVE! md5sum:n8tS5Vn3EwPzpmmIIst24w


# You can replace this text with custom content, and it will be preserved on regeneration
1;
